"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Context = void 0;

function _config() {
  const data = require("@expo/config");

  _config = function () {
    return data;
  };

  return data;
}

function _xdl() {
  const data = require("@expo/xdl");

  _xdl = function () {
    return data;
  };

  return data;
}

function _appleApi() {
  const data = require("../appleApi");

  _appleApi = function () {
    return data;
  };

  return data;
}

function _api() {
  const data = require("./api");

  _api = function () {
    return data;
  };

  return data;
}

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Context {
  constructor() {
    _defineProperty(this, "_hasProjectContext", false);

    _defineProperty(this, "_user", void 0);

    _defineProperty(this, "_manifest", void 0);

    _defineProperty(this, "_apiClient", void 0);

    _defineProperty(this, "_iosApiClient", void 0);

    _defineProperty(this, "_appleCtx", void 0);
  }

  get user() {
    return this._user;
  }

  get hasProjectContext() {
    return this._hasProjectContext;
  }

  get manifest() {
    return this._manifest;
  }

  get api() {
    return this._apiClient;
  }

  get ios() {
    return this._iosApiClient;
  }

  get appleCtx() {
    if (!this._appleCtx) {
      throw new Error('Apple context not initialized.');
    }

    return this._appleCtx;
  }

  hasAppleCtx() {
    return !!this._appleCtx;
  }

  async ensureAppleCtx(options = {}) {
    if (!this._appleCtx) {
      this._appleCtx = await (0, _appleApi().authenticate)(options);
    }
  }

  async init(projectDir, options = {}) {
    const status = await _xdl().Doctor.validateWithoutNetworkAsync(projectDir);

    if (status !== _xdl().Doctor.FATAL) {
      /* This manager does not need to work in project context */
      const {
        exp
      } = (0, _config().getConfig)(projectDir);
      this._manifest = exp;
      this._hasProjectContext = true;
    }

    if (options.allowAnonymous) {
      this._user = (await _xdl().UserManager.getCurrentUserAsync()) || undefined;
    } else {
      this._user = await _xdl().UserManager.ensureLoggedInAsync();
    }

    this._apiClient = _xdl().ApiV2.clientForUser(this.user);
    this._iosApiClient = new (_api().IosApi)(this.user);
  }

}

exports.Context = Context;
//# sourceMappingURL=../__sourcemaps__/credentials/context.js.map
